package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.common.DictTree
import com.gxzc.zen.api.sys.mapper.SysDictMapper
import com.gxzc.zen.api.sys.model.SysDict
import com.gxzc.zen.api.sys.service.ISysDictService
import com.gxzc.zen.common.contants.ZenConstants
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.TreeUtil
import com.gxzc.zen.common.util.common.TreeNode
import com.gxzc.zen.orm.annotation.ZenTransactional
import org.springframework.beans.BeanUtils
import org.springframework.stereotype.Service

/**
 * <p>
 * 字典表 服务实现类
 * </p>
 *
 * @author NorthLan
 * @since 2018-04-28
 */
@Service
class SysDictServiceImpl : ServiceImpl<SysDictMapper, SysDict>(), ISysDictService {

    override fun getRootTree(): TreeNode? {
        val ret = getTree()
        return if (ret.isEmpty()) null else ret[0]
    }

    private fun getTree(): MutableList<DictTree> {
        val nodes = mutableListOf<DictTree>()
        baseMapper.selectList(null)?.forEach {
            val node = DictTree()
            BeanUtils.copyProperties(it, node)
            nodes.add(node)
        }
        return TreeUtil.build(nodes, ZenConstants.TREE_ROOT_PID)
    }


    @ZenTransactional
    override fun insertTree(entity: SysDict) {
        // 没有父节点id 不存在的
        if (entity.parentId == null) {
            entity.parentId = 0L
        }
        // 插入一条,然后修改path
        if (baseMapper.insert(entity) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }

        if (baseMapper.updatePath(entity.parentId!!, entity.id!!) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
    }
}
