package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysResourceRoleMapper
import com.gxzc.zen.api.sys.model.SysResourceRole
import com.gxzc.zen.api.sys.service.ISysResourceRoleService
import com.gxzc.zen.api.sys.service.ISysResourceService
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.orm.annotation.ZenTransactional
import com.gxzc.zen.umps.constant.ZenHttpSession
import com.gxzc.zen.umps.util.ShiroRedisUtil
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * <p>
 * 资源角色表(授权表) 服务实现类
 * </p>
 *
 * @author NorthLan
 * @since 2018-04-28
 */
@Service
class SysResourceRoleServiceImpl : ServiceImpl<SysResourceRoleMapper, SysResourceRole>(), ISysResourceRoleService {
    @Autowired
    private lateinit var sysResourceService: ISysResourceService

    @ZenTransactional
    override fun saveResourceRoleByRoleId(roleId: Long, menuId: Long, resList: List<Long>) {
        // 先获取roleid下的所有对应menuId下的所有resid
        val menuResIdList = sysResourceService.getResourcesByMenuId(menuId).map { it.id!! }
        val ret = baseMapper.selectList(EntityWrapper<SysResourceRole>()
                .eq("role_id", roleId)
                .`in`("resource_id", menuResIdList)).map { it.resourceId!! }
        //
        val deleteIds = ret.subtract(resList)
        val insertIds = resList.subtract(ret)

        if (deleteIds.isEmpty() && insertIds.isEmpty() && resList.isNotEmpty()) {
            throw ZenException(ZenExceptionEnum.BIZ_UPDATE_ERROR)
        }

        var eff = false
        if (deleteIds.isNotEmpty()) {
            // 删除啊 批量啊
            baseMapper.physicalDelete(EntityWrapper<SysResourceRole>().eq("role_id", roleId).`in`("resource_id", deleteIds))
            eff = true
        }
        //
        if (insertIds.isNotEmpty()) {
            val entityList = mutableListOf<SysResourceRole>()
            insertIds.forEach {
                entityList.add(SysResourceRole().apply {
                    this.resourceId = it
                    this.roleId = roleId
                })
            }
            this.insertBatch(entityList)
            eff = true
        }

        if (eff) {
            // 需要更新一下缓存
            ShiroRedisUtil.removeAllSessionsAttributeKey(ZenHttpSession.SESSION_KEY_USER_PERMS)
        }
    }

}
